/*
    ettercap -- the protocol dissector

    Copyright (C) 2001  ALoR <alor@users.sourceforge.net>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "include/ec_main.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "include/ec_dissector.h"
#include "include/ec_inet_structures.h"
#include "include/ec_error.h"

#ifdef DEBUG
   #include "include/ec_debug.h"
#endif

#define ARPBASED 0      // illithid mode from ec_illithid.c
#define IPBASED  1
#define MACBASED 2

typedef struct {
   char mode;
   short proto;
   short port;
   int (*dissector)(u_char *, CONNECTION *);
} DISSECTOR;

static DISSECTOR Available_Dissectors[] = {  {MACBASED,  IPPROTO_TCP,  21, &Dissector_ftp},
                                             //{ARPBASED,  IPPROTO_TCP,  22, &Dissector_ssh},
                                             {MACBASED,  IPPROTO_TCP,  23, &Dissector_telnet},
                                             {MACBASED,  IPPROTO_TCP, 110, &Dissector_pop},
                                          };

//static DISSECTOR Available_Data_Dissectors[] = {  {MACBASED,  IPPROTO_UDP,  4000, &Dissector_Data_icq},
//                                             };


// protos....

void Dissector_Connections( char mode, short proto, u_char *data, CONNECTION *data_to_ettercap  );
void Dissector_Data(short proto, SNIFFED_DATA *data_from_illithid);

// -------------------------------------


void Dissector_Connections( char mode, short proto, u_char *data, CONNECTION *data_to_ettercap )
{

   TCP_header *tcp;
   UDP_header *udp;
   DISSECTOR *ad;

   switch(proto)
   {
      case IPPROTO_TCP:
                        tcp = (TCP_header *) data;

                        for( ad = Available_Dissectors; ad->port != 0; ad++)
                        {
                           if ( ad->proto == IPPROTO_TCP && mode <= ad->mode && (ntohs(tcp->source) == ad->port || ntohs(tcp->dest) == ad->port) )
                              ad->dissector(data, data_to_ettercap);
                        }
                        break;
      case IPPROTO_UDP:
                        udp = (UDP_header *) data;

                        for( ad = Available_Dissectors; ad->port != 0; ad++)
                        {
                           if ( ad->proto == IPPROTO_UDP && mode <= ad->mode && (ntohs(udp->source) == ad->port || ntohs(udp->dest) == ad->port) )
                              ad->dissector(data, data_to_ettercap);
                        }
                        break;
   }


}


/* EOF */
